
var X3DOMObject = function(element,data,options){

    var defaults = {
        debug: true,
        highlight: true,
        fov: 30*Math.PI/180,//deg, vertical
        fov_step: 0.025, //rad
    };

    this._settings = $.extend(defaults,options);
    
    this._FOV = this._settings.fov;
    this._FOV_STEP = this._settings.fov_step;
    this._HIGHLIGHT_ON_MOUSEOVER = this._settings.highlight;
    this._DEBUG = this._settings.debug;
    
    this._resizeTimer = false;
    this._eventCtrlKey = false;
    
    this._X3DOM_SCENE_INIT = this._settings.debug;
    this._X3DOM_SCENE_INIT = false;
    
    this._element = element;
    this.element = element;
     
    this.highlighted_marker_index = null;
    
    this.old_view_translation = null;
    //this.lastMouseX;
    //this.lastMouseY;
    this.data = data;

    this.initialize();
    
};

X3DOMObject.prototype.initialize = function(){
    
    var self = this;

    self.resize();

    //bind resize
    $(window).resize(function(){
        clearTimeout(self.resizeTimer);
        self.resizeTimer = window.setTimeout(function(){
            self.resize();
        },100);
    });

    //self.getX3DOMlibrary();

}

X3DOMObject.prototype.resize = function(){
    
    var self = this;

    var w = $(window).width();
    var h = $(window).height();

    $(self._element).attr("width",w);
    $(self._element).attr("height",h);
  
    var fov = w/h*self._FOV;
    
    //fov = Math.PI/2;
    
    // apply to all viewpoints, but:
    // https://doc.x3dom.org/author/Navigation/Viewpoint.html
    // = correct field of view is not guaranteed in X3DOM
    $(self._element).find("Viewpoint").each(function(){
        $(this).attr("fieldOfView",fov);
    });
    
    self.data.camera.fov = fov;
    
    if (Map.marker != undefined){
        Map.marker.setFoV(fov);
    }
}

X3DOMObject.prototype.getX3DOMlibrary = function(){
    
    var self = this;
    
    $.getScript("js/x3dom-full.debug.js",function(){
        //wait until it parsed the DOM
        x3dom.runtime.ready = function(){
            self.deepInit();
        };
    });
}

// so one doesn't have to click to enable keydown/keyup
X3DOMObject.prototype.focusOnCanvas = function(){
    
    this.element.getElementsByTagName("canvas")[0].focus();
    
}

X3DOMObject.prototype.FoVEvents = function(){
    
    var self = this;
    // true = CAPTURING = from parent to child
    // canvas is a child of x3d
    
    //Chrome?!
    self._element.addEventListener('mousewheel',function(e){
        var delta = (e.deltaY>0)?self._FOV_STEP:-self._FOV_STEP;
        self.changeFoV(e,delta);
    },true);
    
    //Firefox?!
    self._element.addEventListener('DOMMouseScroll',function(e){  
        var delta = (e.detail>0)?self._FOV_STEP:-self._FOV_STEP;
        self.changeFoV(e,delta);
    },true);
    
    
}


X3DOMObject.prototype.changeFoV = function(e,val){
    
    var self = this;
    
    //if (event.ctrlKey) {

        fov = self.getFoV()+val;
        self.setFoV(fov);
        Map.marker.setFoV(fov);

        e.preventDefault();
        e.stopPropagation();
        return false;

    //}
    
}

X3DOMObject.prototype.setFoV = function(val){
  
    var vp = $(this._element).find("Viewpoint")[0];
    $(vp).prop("fieldOfView",val);
  
}

X3DOMObject.prototype.getFoV = function(){
  
    var vp = $(this._element).find("Viewpoint")[0];
    return parseFloat($(vp).prop("fieldOfView"));
    
}

X3DOMObject.prototype.KeyEvents = function(){
    
    var self = this;
    //Chrome?!
    self._element.addEventListener('mousemove',function(e){
        if (e.ctrlKey){
            self._ctrlKey = true;
        }else{
            self._ctrlKey = false;
        }
        if (e.shiftKey){
            self._shiftKey = true;
        }else{
            self._shiftKey = false;
        }        
    },true);

}
/*
X3DOMObject.prototype.mouseMove = function(e){

    //offsetX / offsetY polyfill for FF
    var target = e.target || e.srcElement;
    var rect = target.getBoundingClientRect();

    e.offsetX = e.clientX - rect.left;
    e.offsetY = e.clientY - rect.top;

    if (this.lastMouseX === -1){
        this.lastMouseX = e.offsetX;
    }
    if (this.lastMouseY === -1){
        this.lastMouseY = e.offsetY;
    }

    if (this.draggedTransformNode){
        this.dragMarker(e.offsetX - this.lastMouseX, e.offsetY - this.lastMouseY);
    }

    this.lastMouseX = e.offsetX;
    this.lastMouseY = e.offsetY;
    
}



X3DOMObject.prototype.dragMarker = function(dx,dy){
    
    var offsetUp    = draggingUpVec.multiply(-dy);
    var offsetRight = draggingRightVec.multiply(dx);

    this.unsnappedDragPos = this.unsnappedDragPos.add(offsetUp).add(offsetRight);

    $(this.draggedTransformNode).attr("translation", this.unsnappedDragPos.toString());
    
    //use callback?
    
}
*/
X3DOMObject.prototype.stopDraggingMarker = function(e){
    
    this.removeEventListener('mousemove',this.mouseMove,true);
    this.removeEventListener('mouseup',this.mouseup,true);
    
    draggedTransformNode = null;
    draggingUpVec        = null;
    draggingRightVec     = null;
    unsnappedDragPos     = null;
    
    document.getElementById("navInfo").setAttribute("type",'"examine"');
    
}

X3DOMObject.prototype.createInfoWindow = function(){

    var el = $("<div>",{id:"infowindow"}).css({
        position:"absolute",
        left:"2px"
    });
    $("body").append(el);
    
    $(this.element).find("canvas").on('mousemove',function(e){
        $("#infowindow").css({
            position:"absolute",
            top: (e.originalEvent.clientY+15)+"px",
            left: (e.originalEvent.clientX+15)+"px"
        });
    });

}

X3DOMObject.prototype.resetInfoWindow = function(){

    var msg = "<div>x=&ndash;</div>"+
                "<div>y=&ndash;</div>"+
                "<div>z=&ndash;</div>"+
                "<div>d=&infin;</div>";
    
    msg = "";
    
    //this.showMessage("infowindow",msg);
    this.hideMessage("infowindow");
    

}

X3DOMObject.prototype.createShapeTitle = function(){

    var el = $("<div>",{id:"shapetitle"}).css({
        position:"absolute",
        bottom:"2px",
        right:"2px",
        "z-index":10,
        background:"black",
        border:"1px solid rgba(200,200,200,0.5)",
        color:"white",
        padding:"0px"
    });
    $("body").append(el);

}

X3DOMObject.prototype.showMessage = function(id,msg){

    $("#"+id).show();
    
    $("#"+id).html($("<div>").html(msg).css({
        padding:"5px"
    })).show();

}

X3DOMObject.prototype.hideMessage = function(id){

    $("#"+id).hide();

}


X3DOMObject.prototype.registerShapesEvents = function(){
    
    var self = this;

    var tmp = new X3DOMObject.PointerMarker();
    
    var inlines = $(self._element).find("inline");
    
    inlines.each(function(){

        if ($(this).attr("name")!="back"){

            var shapes = $(this).find("Shape");

            shapes.each(function(){
            
                this.addEventListener('click',function(e){
                    //console.log(e.worldX+" "+e.worldY+" "+e.worldZ);
                    //createSphere(e.worldX,e.worldY,e.worldZ);
                    if (self._ctrlKey){
                        self.placeMarker(e.worldX,e.worldY,e.worldZ);
                    }
                    
                });
            
                $(this).on("mousemove",function(e){
    
                    //console.log(e);
                    
                    var x = e.originalEvent.worldX;
                    var y = e.originalEvent.worldY;
                    var z = e.originalEvent.worldZ;

                    if (self._ctrlKey||SETTINGS.pointer){
                        //self.placeMarker(x,y,z,"sliding_sphere");
                        X3DOMObject.Marker.place(x,y,z,"sliding_sphere");
                        $("#sliding_sphere").find("switch").attr("whichChoice",0);
                    }else{
                        //self.placeMarker(0,0,0,"sliding_sphere");
                        X3DOMObject.Marker.place(0,0,0,"sliding_sphere");
                        $("#sliding_sphere").find("switch").attr("whichChoice",-1);
                    }
                    
                    var v = new x3dom.fields.SFVec3f(x,y,z);
                    
                    //for printing
                    var x = v.x.toFixed(2);
                    var y = v.y.toFixed(2);
                    var z = v.z.toFixed(2);
                    var l = (v.length()).toFixed(1);
                    
                    var msg = "<div>x="+x+"</div>"+
                                "<div>y="+y+"</div>"+
                                "<div>z="+z+"</div>"+
                                "<div>d="+l+"</div>";
                    
                    msg = "<div>"+l+" m</div>";
                                
                    //self.showMessage("infowindow",msg);

                });
                
                $(this).on("click",function(e){
                    if ((self._shiftKey)||(SETTINGS.highlight)){
                        self.toggleShape(this);
                    }
                });
                
                $(this).on("mouseover",function(e){
                
                    var tmp = $(this).attr("id");
              
                    if (typeof tmp === 'undefined'){
                        tmp = "missing <b>id</b> attribute";
                    }

                    x3dom.debug.logInfo(tmp);
                    self.showMessage("shapetitle",tmp);
                
                    // e.ctrlKey will not work because X3DOM does something to events
                    if (self._shiftKey){
            
                        if (self._HIGHLIGHT_ON_MOUSEOVER){
                            self.highlightShape(this);
                        }

                    }
            
                });
            
                $(this).on("mouseout",function(e){
                    
                    self.dehighlightShape(this);
                    //self.placeMarker(0,0,0,"sliding_sphere");
                    self.hideMessage("shapetitle");
                    //self.resetInfoWindow();

                });
                
                $(this).on("keydown",function(e){
                    
                    console.log("KeyDown");
                    self.highlightShape(this);
                    
                });
                
                this.addEventListener('keydown',function(e){
                    console.log("Crack!");
                });
                
                
                $(this).on("keyup",function(e){
                    
                    console.log("keyup");
                    self.highlightShape(this);
                    
                });                
            
            });
        
        }

    });
    
}

X3DOMObject.prototype.highlightShape = function(elem){
    
    var m = $(elem).find("Material[class='hl']");
    
    if (m.length==0){
        m = $("<Material>",{class:'hl'});
        //m.attr("ambientintensity","1");
        //m.attr("shininess","0.2");
        m.attr("emissiveColor","0.1 0.3 0.1");
        $(elem).find("Appearance").append(m);
    }

}

X3DOMObject.prototype.dehighlightShape = function(elem){
    
    var self = this;
    
    $(elem).find("Appearance").find("Material[class='hl']").remove();
    
    m = $(elem).find("Material[class='sl']");
    if (m.length==1){
        self.selectShape(elem);
    }
  
}

X3DOMObject.prototype.toggleShape = function(elem){

    var self = this;
    
    var m = $(elem).find("Material[class='sl']");
        
    if (m.length!=0){
        self.deselectShape(elem);
    }else{
        self.selectShape(elem);
    }
  
}

X3DOMObject.prototype.selectShape = function(elem){
    
  var self = this;
    
  self.deselectShape(elem);
  
  var m = $(elem).find("Material[class='sl']");
  
  m = $("<Material>",{class:'sl'});
  m.attr("emissiveColor","1 1 1");
  //m.attr("transparency","0");
  $(elem).find("Appearance").append(m);
  
}

X3DOMObject.prototype.deselectShape = function(elem){

  var tmpapp = $(elem).find("Appearance").find("Material[class='sl']").remove();

}

X3DOMObject.prototype.createMarker = function(x,y,z,id){

    var self = this;
    
    sph_class = "";
    
    var index = null;
    
    if ((id=="")||(id==undefined)){
        sph_class = "my-markers";
        index = $("."+sph_class).length;
        id = "my-sph-"+index;
    }

    var html = `
    <group id='`+id+`' class='`+sph_class+`'>
    <switch whichChoice='0'>
    <transform translation='`+x+` `+y+` `+z+`' rotation='0 0 0 0'>
        <shape> 
        <appearance> 
            <material diffuseColor='0.07 1 0.07' transparency='0.0'></material> 
        </appearance> 
        <Sphere DEF="sphere" radius="1" />
        </shape> 
    </transform>
    </switch>
    </group>
    `;
    
    var sphere_element = $(html);
    
    $(this._element).find("scene").append(sphere_element);
    
    var shape = $(sphere_element).find("shape");
    var id_prefix = $(sphere_element).attr("id").substr(0,7);
    
    return sphere_element;
    // sphere events
    // Drag, select, delete
    // Data.push()
    
}

X3DOMObject.prototype.updateMarkersIndices = function(){
    
    $(this.element).find(".my-markers").each(function(i,v){

        $(this).attr("id","my-sph-"+i);

    });
    // sphere events
    // Drag, select, delete
    // Data.push()
    
}

X3DOMObject.prototype.placeMarker = function(x,y,z,id){

    var tr = $("#"+id).find("transform");  
    $(tr).attr("translation",x+" "+y+" "+z);

}

X3DOMObject.prototype.highlightMarker = function(index){
    
    var color = "1 0.5 0.5";
    
    $("#my-sph-"+index).find('material').attr("diffuseColor",color);
    
}

X3DOMObject.prototype.dehighlightMarker = function(index){

    var elem = $("#my-sph-"+index).find('material');
    var color = "0.07 1 0.07";

    if (!elem.prop("selected")){
        elem.attr("diffuseColor",color);
    }

}

X3DOMObject.prototype.toggleMarker = function(index){

    var elem = $("#my-sph-"+index).find('material');
    var color = elem.attr("diffuseColor");

    if (elem.prop("selected")){
        elem.prop("selected",false);
    }else{
        elem.prop("selected",true);
    }

}

X3DOMObject.prototype.getViewDirection = function(){
    
    var elem = this.element;
    
    var vp_mat = elem.runtime.viewMatrix();

    //console.log(vp_mat);
    
    var vMatInv  = vp_mat.inverse();
    var viewDir  = vMatInv.multMatrixVec(new x3dom.fields.SFVec3f(0.0, 0.0, -1.0));
    
    var angle = Math.atan2(viewDir.x,-viewDir.z)*180/Math.PI;
    
    return angle;
    
}

X3DOMObject.prototype.getViewTranslation = function(){

    var elem = this.element;

    var vp_mat = elem.runtime.viewMatrix().inverse();
    var vp_rotation = new x3dom.fields.Quaternion(0, 0, 1, 0);

    vp_rotation.setValue(vp_mat);

    var vp_translation = vp_mat.e3();

    return vp_translation;

}

// Marker
X3DOMObject.Marker = function(x,y,z){
    
    this._x = x || 0;
    this._y = y || 0;
    this._z = z || 0;
    
    this.init();
    
}

X3DOMObject.Marker.prototype.init = function(){
    
    this._elem = Scene.createMarker(this._x,this._y,this._z);
    this._shape = this._elem.find("shape");
    this._registerEvents();
    
}




X3DOMObject.Marker.prototype._registerEvents = function(){
    
    var PREFIX = "my-sph-";
    
    var self = this;
    var marker = this._shape;
    var id_prefix = $(this._elem).attr("id").substr(0,PREFIX.length);
    var index = parseInt($(this._elem).attr("id").substr(PREFIX.length));
    
    marker.on('click',function(e){
        
        //self.dehighlight();
        //X3DOMObject.PointerMarker.dehighlight(self._elem);
        
        if (Scene._ctrlKey){

            if (id_prefix==PREFIX){

                self._elem.remove();
                
                // REMOVE
                Data.markers.splice(index);
                Scene.updateMarkersIndices();
                
                Map.deleteMarker(index);
                
            }

        }else{

            Scene.toggleMarker(index);
            Map.toggleMarker(index);

        }
        
    });
    
    marker.on('mouseover',function(e){
        
        X3DOMObject.Marker.highlight(self._elem);
        Map.highlightMarker(index);
        Scene.highlighted_marker_index = index;
        
        if (Scene._ctrlKey){
            if (id_prefix==PREFIX){
                $("#sliding_sphere").find("switch").attr("whichChoice",-1);
            }
        }
        
    });

    /*
    marker.addEventListener('mouseover',function(){
        
        Scene.showMessage("infowindow","X");
        
    },true);
    */
    
    marker.on('mouseout',function(e){
        
        //self.highlightMarker(index);
        Scene.highlighted_marker_index = null;

        if (Scene.draggedTransformNode==null){
            
            if (!self._elem.prop("selected")){
                X3DOMObject.Marker.dehighlight(self._elem);
                Map.dehighlightMarker(index);
            }else{
                X3DOMObject.Marker.highlight(self._elem);
                Map.highlightMarker(index);
            }

        }
        
    });
    
    marker.on('mousedown',function(e){

        X3DOMObject.Marker.dehighlight(self._elem);
        Map.dehighlightMarker(index);
        
        document.getElementById("navInfo").setAttribute("type", '"NONE"');
        
        Scene.lastMouseX = event.offsetX;
        Scene.lastMouseY = event.offsetY;
        
        X3DOMObject.Marker.dragStart(this);

        //$(this).on('mousemove',x3d_markerDrag);
        /*
        $(Scene.element).on('mousemove',function(){
            console.log("preMouseMove");
            x3d_sceneMouseMove2();
        },true);
        */

        Scene.element.addEventListener('mousemove',X3DOMObject.Marker.mouseMove,true);
        Scene.element.addEventListener('mouseup',X3DOMObject.Marker.mouseUp,true);
                
    });
    
    marker.on('mouseup',function(e){

        X3DOMObject.Marker.highlight(self._elem);
        Map.highlightMarker(index);

    });
    
}

X3DOMObject.Marker.mouseUp = function(){

    var elem = Scene.draggedTransformNode.parent().parent();

    if (!Scene.markerdragged){
        X3DOMObject.Marker.toggle(elem);
    }
    
    Scene.element.removeEventListener('mousemove',X3DOMObject.Marker.mouseMove,true);
    Scene.element.removeEventListener('mouseup',X3DOMObject.Marker.mouseUp,true);
    
    Scene.draggedTransformNode = null;
    Scene.draggingUpVec        = null;
    Scene.draggingRightVec     = null;
    Scene.unsnappedDragPos     = null;
    Scene.markerdragged = false;
    
    document.getElementById("navInfo").setAttribute("type",'"examine"');
    
}

// from https://x3dom.org/x3dom/example/MovingObjectsWithDOMEvents.html
X3DOMObject.Marker.dragStart = function(elem){

    // move up from <shape> to <transform>
    var transformNode = $(elem).parent();
    var tr0 = $(transformNode).attr("translation");

    Scene.draggedTransformNode = transformNode; 

    Scene.unsnappedDragPos = new x3dom.fields.SFVec3f.parse(tr0);
    
    //compute the dragging vectors in world coordinates
    //(since navigation is disabled, those will not change until dragging has been finished)
    
    var vMatInv = Scene.element.runtime.viewMatrix().inverse();
    var viewDir = vMatInv.multMatrixVec(new x3dom.fields.SFVec3f(0.0, 0.0, -1.0));

    //use the viewer's up-vector and right-vector
    Scene.draggingUpVec    = vMatInv.multMatrixVec(new x3dom.fields.SFVec3f(0.0, 1.0,  0.0));
    Scene.draggingRightVec = viewDir.cross(Scene.draggingUpVec);

    var p1 = Scene.element.runtime.calcCanvasPos(Scene.unsnappedDragPos.x, Scene.unsnappedDragPos.y, Scene.unsnappedDragPos.z);
    var p2 = Scene.element.runtime.calcCanvasPos(Scene.unsnappedDragPos.x + Scene.draggingRightVec.x,
                                         Scene.unsnappedDragPos.y + Scene.draggingRightVec.y,
                                         Scene.unsnappedDragPos.z + Scene.draggingRightVec.z);

    var magnificationFactor = 1.0 / Math.abs(p1[0] - p2[0]);

    //scale up vector and right vector accordingly            
    Scene.draggingUpVec    = Scene.draggingUpVec.multiply(magnificationFactor);
    Scene.draggingRightVec = Scene.draggingRightVec.multiply(magnificationFactor);
    
}

X3DOMObject.Marker.mouseMove = function(event){
    
    Scene.markerdragged = true;
    
    X3DOMObject.Marker.highlight(Scene.draggedTransformNode.parent().parent());
    
    //offsetX / offsetY polyfill for FF
    var target = event.target || event.srcElement;
    var rect = target.getBoundingClientRect();

    event.offsetX = event.clientX - rect.left;
    event.offsetY = event.clientY - rect.top;

    if (Scene.lastMouseX === -1){
        Scene.lastMouseX = event.offsetX;
    }
    if (Scene.lastMouseY === -1){
        Scene.lastMouseY = event.offsetY;
    }

    if (Scene.draggedTransformNode){
        X3DOMObject.Marker.drag(event.offsetX - Scene.lastMouseX, event.offsetY - Scene.lastMouseY);
    }

    Scene.lastMouseX = event.offsetX;
    Scene.lastMouseY = event.offsetY;
    
}

X3DOMObject.Marker.drag = function(dx,dy){
 
    var offsetUp    = Scene.draggingUpVec.multiply(-dy);
    var offsetRight = Scene.draggingRightVec.multiply(dx);

    Scene.unsnappedDragPos = Scene.unsnappedDragPos.add(offsetUp).add(offsetRight);

    $(Scene.draggedTransformNode).attr("translation", Scene.unsnappedDragPos.toString());

    var sphere = $(Scene.draggedTransformNode).parent().parent();
    var index = parseInt(sphere.attr("id").substr(7));

    var c = Data.markers[index];
    
    c.x = Scene.unsnappedDragPos.x;
    c.y = Scene.unsnappedDragPos.y;
    c.z = Scene.unsnappedDragPos.z;
    
    var azimuth = Math.atan2(c.x,-c.z)*180/Math.PI;
    //var initial_heading = Data.camera.heading;
    var distance = Math.sqrt(c.x*c.x+c.z*c.z);
    
    var p1_ll = Map.marker._latlng;
    var p2_ll = p1_ll.CoordinatesOf(azimuth+INIT_HEADING_TMP,distance);
    
    c.latitude = p2_ll.lat;
    c.longitude = p2_ll.lng;
    c.altitude = c.y;
    
    Map.marker.moveMeasureMarker(p2_ll,index);
    
}

X3DOMObject.Marker.place = function(x,y,z,id){

    var tr = $("#"+id).find("transform");  
    $(tr).attr("translation",x+" "+y+" "+z);

}

X3DOMObject.Marker.highlight = function(elem){
    
    var color = "1 0.5 0.5";
    elem.find('material').attr("diffuseColor",color); 
    
}

X3DOMObject.Marker.dehighlight = function(elem){
    
    var color = "0.07 1 0.07";
    elem.find('material').attr("diffuseColor",color);
    
}

X3DOMObject.Marker.toggle = function(elem){
    
    if (elem.prop("selected")){
        elem.prop("selected",false);
    }else{
        elem.prop("selected",true);
    }
    
}
// PointerMarker

X3DOMObject.PointerMarker = function(){
  
    //init
    this._init();
  
};

X3DOMObject.PointerMarker.prototype._init = function(){

    this._elem = Scene.createMarker(0,0,0,"sliding_sphere");;
    this._shape = this._elem.find("shape");
    this._registerEvents();
    
    this._elem.find("switch").attr("whichChoice",-1);
    
}

X3DOMObject.PointerMarker.prototype._registerEvents = function(){
    
    var self = this;
    var Camera = Map.marker;

    self._shape.on("click",function(e){
        
        var xyz = $(this).parent().attr("translation");
        xyz = xyz.split(" ");
        
        X3DOMObject.Marker.dehighlight(self._elem);
        //Scene.createMarker(xyz[0],xyz[1],xyz[2]);

        var index = Data.markers.length;
        
        // Update data
        var mark = new X3L({
            x: parseFloat(xyz[0]) || 0,
            y: parseFloat(xyz[1]) || 0,
            z: parseFloat(xyz[2]) || 0
        });
        Data.markers.push(mark);

        //console.log(mark);

        // create marker on the scene
        new X3DOMObject.Marker(mark.x,mark.y,mark.z);

        // create marker on the map
        var azimuth = Math.atan2(mark.x,-mark.z)*180/Math.PI;
        var distance = Math.sqrt(mark.x*mark.x+mark.z*mark.z);

        Camera.createMeasureMarker(azimuth+INIT_HEADING_TMP,distance);
        
        var map_mark = Camera._measureMarkers[Camera._measureMarkers.length-1];
        
        mark.latitude = map_mark._latlng.lat;
        mark.longitude = map_mark._latlng.lng;
        
        X3DOMObject.MapMarker.registerEvents(map_mark);
        
        Camera._syncMeasureMarkersToBasePoint();

    });
    
    self._shape.on("mousedown",function(e){

        X3DOMObject.Marker.highlight(self._elem);

    });
    
}

X3DOMObject.MapMarker = {};

X3DOMObject.MapMarker.registerEvents = function(map_mark){
    
        var Camera = Map.marker;
    
        map_mark.on('mouseover',function(){
            
            var elem = $("#my-sph-"+this._index);
            //console.log(this._index);
            X3DOMObject.Marker.highlight(elem);
            Map.highlightMarker(this._index);
            
        });
        
        map_mark.on('mouseout',function(){
            
            if (Camera.draggedMarker._index!=this._index){
                var elem = $("#my-sph-"+this._index);
                X3DOMObject.Marker.dehighlight(elem);
                Map.dehighlightMarker(this._index);
            }

        });
        
        map_mark.on('click',function(e){
            
            var elem = $("#my-sph-"+this._index);
            X3DOMObject.Marker.toggle(elem);
            Map.toggleMarker(this._index);
            
            //if ctrl - remove
            //Data.markers.splice(index);
            //Scene.updateMarkersIndices();

        });
        
        map_mark.on('mousedown',function(){
            
            var elem = $("#my-sph-"+this._index);
            X3DOMObject.Marker.dehighlight(elem);
            Map.dehighlightMarker(this._index);
            
        });
    
        map_mark.on('mouseup',function(e){
            
            var index = this._index;
            var elem = $("#my-sph-"+index);
                    
            if (e.originalEvent.ctrlKey){
                
                Data.markers.splice(index);
                
                Map.deleteMarker(index);
                
                elem.remove();
                Scene.updateMarkersIndices();
                
            }else{
            
                X3DOMObject.Marker.highlight(elem);
                Map.highlightMarker(index);
            
            }
            
        });
    
}









