
var Data = {
    camera:{},
    markers:[],
};

var Scene;
var Map;

var INIT_HEADING = 0;
var INIT_HEADING_TMP = 0;

var SETTINGS = {
    'pointer': false,
    'highlight': false,
    'moreinfo': false
}

$(function(){
    
    parseURL();
    
    title_init();
    help_init();
    menu_init();
    light_init();

    //x3dm.update(X3L);
    //lft.update(X3L);
    
    //x3dm.get(X3L);
    //lft.get(X3L);
    
});

function title_init(){

    var html = $("<div>").html("Demo");

    html.css({
        position:"absolute",
        top: "10px",
        left: "50%",
        color:"white",
        "font-size":"40px",
        "font-weight":"bold",
        "font-family": '"Helvetica Neue", Helvetica, Arial, sans-serif',
        "user-select": "none"
    });
    
    $("body").append(html);

}

function light_init(){
  
    $.ajax({
        url: "kml/test.kml",
        success: function(response){

            var longitude = parseFloat($(response).find("Camera").find("longitude").text());
            var latitude  = parseFloat($(response).find("Camera").find("latitude").text());
            var altitude  = parseFloat($(response).find("Camera").find("altitude").text());
            
            var heading = parseFloat($(response).find("Camera").find("heading").text());
            var tilt    = parseFloat($(response).find("Camera").find("tilt").text());
            var roll    = parseFloat($(response).find("Camera").find("roll").text());
            
            var fov    = parseFloat($(response).find("Camera").find("fov").text());
            
            Data.camera = new X3L({
                x: 0,
                y: 0,
                z: 0,
                latitude: latitude || 0,
                longitude: longitude || 0,
                altitude: altitude || 0,
                heading: heading || 0,
                tilt: tilt || 0,
                roll: roll || 0,
                fov: fov || 0,
            });
            
            //INIT_HEADING = heading;
            INIT_HEADING_TMP = heading;
            
            var element = document.getElementById('x3d_id');
            Scene = new X3DOMObject(element,Data,{});
            
            $.getScript("js/x3dom-full.debug.js",function(){

                Map = new LeafletObject('leaflet_map',Data,{});
                //wait until it parsed the DOM
                x3dom.runtime.ready = function(){
                    map_resize_init();
                    deepInit();
                    //x3d_initial_camera_placement();
                };
            });
            
        },
    });

}

function map_resize_init(){
    
    var html = $("<div>",{id:"map_resizer_handle"});
    
    $("#map_wrapper").append(html);
    
    html.on("mousedown",function(){
        
        $("body").on("mousemove",mouseMove_resizeMap);
        $(Scene.element).find("canvas").on("mousemove",mouseMove_resizeMap);
        
    });
    
    html.on("mouseup",function(){
        
        $("body").off("mousemove",mouseMove_resizeMap);
        $(Scene.element).find("canvas").off("mousemove",mouseMove_resizeMap);

    });
    
}

function mouseMove_resizeMap(e){
    
    var xm = e.clientX;
    var ym = e.clientY;

    var xw = $(window).width();
    var yw = $(window).height();

    var xb = parseInt($("#map_wrapper").css("right"));
    var yb = parseInt($("#map_wrapper").css("bottom"));

    var w = xw - xm - xb;
    var h = yw - ym - yb;

    $("#map_wrapper").width(w);
    $("#map_wrapper").height(h);

    $("#leaflet_map").width(w);
    $("#leaflet_map").height(h);

    setTimeout(function(){ Map._map.invalidateSize()}, 100);
    
}

function deepInit(){

    Scene.FoVEvents();
    Scene.KeyEvents();

    Scene._element.runtime.enterFrame = function(){

        // the only <strong> in the document
        var progress_element = $(Scene._element).find("strong");
        var progress_counter = $(progress_element).html();
        progress_counter = progress_counter.split(" ");
        cnt = parseInt(progress_counter[1]);

        if (!Scene._X3DOM_SCENE_INIT&&(cnt==0)){

            if(Scene._DEBUG){
                Scene.createInfoWindow();
                Scene.createShapeTitle();
                //createSphere(0,0,0,"sliding_sphere");
                Scene.registerShapesEvents();
            }

            Scene._X3DOM_SCENE_INIT = true;

            x3d_events();
            leaf_events();
        }
    };
    
}

function x3d_initial_camera_placement(){
    
    // Roll compensation
    var heading = Data.camera.heading*Math.PI/180;
    var tilt = (Data.camera.tilt-90)*Math.PI/180;
    var roll = Data.camera.roll*Math.PI/180;
    
    // Altitude is relative. Do not care.

    // Roll
    var z = new x3dom.fields.SFVec3f(0,0,1);
    var qr = x3dom.fields.Quaternion.axisAngle(z,-roll);
    var M_roll = qr.toMatrix();

    // Tilt
    var x = new x3dom.fields.SFVec3f(1,0,0);
    var qt = x3dom.fields.Quaternion.axisAngle(x,-tilt);
    var M_tilt = qt.toMatrix();

    // Heading
    var y = new x3dom.fields.SFVec3f(0,1,0);
    var qh = x3dom.fields.Quaternion.axisAngle(y,-heading);
    var M_heading = qh.toMatrix();

    var M = M_heading.mult(M_tilt).mult(M_roll);
    
    // store matrices
    Data.camera.Matrices = {
        Head: M_heading,
        Tilt: M_tilt,
        Roll: M_roll,
        R: M
    };
    
    // set view
    var Q = new x3dom.fields.Quaternion(0, 0, 1, 0);
    Q.setValue(M.inverse());
    var AA = Q.toAxisAngle();
    
    var viewpoint = $(Scene.element).find("Viewpoint");
    viewpoint.attr("orientation",AA[0].toString()+" "+AA[1]);
    
}

function x3d_events(){

    var elem = Scene.element;

    elem.addEventListener('keydown',function(e){
        
        if ((e.key=="Shift")||(SETTINGS.highlight&&!SETTINGS.pointer)){
            // select shape
            var sr = elem.runtime.shootRay(e.path[0].mouse_drag_x,e.path[0].mouse_drag_y);
            Scene.highlightShape(sr.pickObject);
        }
        
        if (e.key=="Control"){
            
            var x,y,z;
            var dist = 1000;
            var sr = elem.runtime.shootRay(e.path[0].mouse_drag_x,e.path[0].mouse_drag_y);
            
            if (sr.pickPosition != null){
                
                var index = Scene.highlighted_marker_index;
                
                if ((index==null)||(Data.markers[index]==undefined)){
                
                    x = sr.pickPosition.x;
                    y = sr.pickPosition.y;
                    z = sr.pickPosition.z;
                    
                }else{
                    
                    x = Data.markers[index].x;
                    y = Data.markers[index].y;
                    z = Data.markers[index].z;
                    
                }

                dist = Math.sqrt(x*x+z*z);

                X3DOMObject.Marker.place(x,y,z,"sliding_sphere");
                $("#sliding_sphere").find("switch").attr("whichChoice",0);
    
            }

            var vr = elem.runtime.getViewingRay(e.path[0].mouse_drag_x,e.path[0].mouse_drag_y);
            
            var angle = Math.atan2(vr.dir.x,-vr.dir.z)*180/Math.PI;
            //var initial_heading = Data.camera.heading;
            var initial_heading = INIT_HEADING_TMP;

            Map.marker.placeSlidingMarker(angle+initial_heading,dist);
            
        }
        
    },true);
    
    elem.addEventListener('keyup',function(e){
        
        if (e.key=="Shift"){
            // select shape
            var sr = elem.runtime.shootRay(e.path[0].mouse_drag_x,e.path[0].mouse_drag_y);
            Scene.dehighlightShape(sr.pickObject);
        }
        
        if (e.key=="Control"){
            
            X3DOMObject.Marker.place(0,0,0,"sliding_sphere");
            $("#sliding_sphere").find("switch").attr("whichChoice",-1);
            
            Map.marker.removeSlidingMarker();
            
        }
        
    },true);
    
    elem.addEventListener('mousedown',function(){
        elem.addEventListener('mousemove',x3d_mouseMove,true);
    });

    elem.addEventListener('mouseup',function(){
        elem.removeEventListener('mousemove',x3d_mouseMove,true);
    });

    elem.addEventListener('mousemove',function(e){
        
        // have to focus if want key events to work w/o extra click
        Scene.focusOnCanvas();
        
        var dist = 1000;

        var sr = elem.runtime.shootRay(e.clientX,e.clientY);

        if (sr.pickPosition != null){
            
            var index = Scene.highlighted_marker_index;
            var x,y,z;
            
            if ((index==null)||(Data.markers[index]==undefined)){
            
                x = sr.pickPosition.x;
                y = sr.pickPosition.y;
                z = sr.pickPosition.z;
                
            }else{
                
                x = Data.markers[index].x;
                y = Data.markers[index].y;
                z = Data.markers[index].z;
                
            }

            dist = Math.sqrt(x*x+z*z);
            var dist2 = Math.sqrt(y*y+dist*dist);
            
            var dist_msg = "";
            var id_msg = "";
            
            if (SETTINGS.moreinfo){
                var st = $(sr.pickObject).attr("id");
                if (typeof st === 'undefined'){
                    id_msg = "n/a";
                }else{
                    id_msg = st;
                }
                
                dist_msg += "<b>shape id:</b> "+id_msg+"<br/>";
                dist_msg += "<b>x</b> = "+x.toFixed(2)+"<br/>";
                dist_msg += "<b>y</b> = "+y.toFixed(2)+"<br/>";
                dist_msg += "<b>z</b> = "+z.toFixed(2)+"<br/>";
                dist_msg += "<b>d<sub>xz</sub></b> = "+dist.toFixed(2)+" m<br/>";
                dist_msg += "<b>d<sub>xyz</sub></b> = "+dist2.toFixed(2);
            }else{
                dist_msg = dist.toFixed(1);
            }
            
        }else{

            dist_msg = "&infin;";

        }
        
        var msg = "<div>"+dist_msg+" m</div>";
        
        Scene.showMessage("infowindow",msg);
        
        if (SETTINGS.highlight&&!SETTINGS.pointer){
            
            var sr = elem.runtime.shootRay(e.path[0].mouse_drag_x,e.path[0].mouse_drag_y);
            Scene.highlightShape(sr.pickObject);
            
        }
        
        if ((Scene._ctrlKey)||(SETTINGS.pointer)){

            // show shadow marker
            
            var vr = elem.runtime.getViewingRay(e.clientX,e.clientY);

            var angle = Math.atan2(vr.dir.x,-vr.dir.z)*180/Math.PI;
            //var initial_heading = Data.camera.heading;
            var initial_heading = INIT_HEADING_TMP;

            Map.marker.placeSlidingMarker(angle+initial_heading,dist);

        }else{

            // hide shadow marker
            Map.marker.removeSlidingMarker();

        }
        
        if (e.buttons==1){
            // upright view
            var mat = Scene.element.runtime.viewMatrix().inverse();
            
            var from = mat.e3();
            var at = from.subtract(mat.e2());

            var up = new x3dom.fields.SFVec3f(0, 1, 0);
            
            var s = mat.e2().cross(up).normalize();
            
            var newup = mat.e2().cross(s).normalize().negate();
            
            //at = from.add(v);
            
            mat = x3dom.fields.SFMatrix4f.lookAt(from, at, newup);
            //mat = mat.inverse();
            
            var Q = new x3dom.fields.Quaternion(0,0,1,0);
            Q.setValue(mat);
            var AA = Q.toAxisAngle();
    
            var viewpoint = $(Scene.element).find("Viewpoint");
            viewpoint.attr("orientation",AA[0].toString()+" "+AA[1]);
        }
        
        
        // what is this?
        //x3d_mouseMove();
        
    },true);

    elem.addEventListener('mouseout',function(e){
        
        // hide shadow marker
        Map.marker.removeSlidingMarker();
        Scene.hideMessage("infowindow");
        
        
    });

}

function leaf_events(){
    
    var Camera = Map.marker;
    
    Camera._map.on('click',function(e){

        if (e.originalEvent.ctrlKey){

            var Lm = Camera._measureMarkers[Data.markers.length];

            if (Lm!=undefined){
            
                var mark = new X3L({
                    latitude: Lm._latlng.lat,
                    longitude: Lm._latlng.lng
                });
                
                var p1_ll = Camera._latlng;
                var p2_ll = Lm._latlng;

                //var azimuth = getAzimuth(p1_ll,p2_ll);
                var azimuth = getAzimuth(p1_ll,p2_ll);
                
                //console.log(azimuth);
                //console.log(azimuth2);
                
                var initial_heading = INIT_HEADING_TMP;
                
                var angle = azimuth - initial_heading;
                var distance = p1_ll.distanceTo(p2_ll);
                
                //console.log("angle from lat lng: "+angle);
                
                mark.x = distance*Math.sin(Math.PI/180*angle);
                mark.y = 0;
                mark.z = -distance*Math.cos(Math.PI/180*angle);
                
                Data.markers.push(mark);
                
                new X3DOMObject.Marker(mark.x,mark.y,mark.z);
                
                //Scene.createMarker(mark.x,mark.y,mark.z);
                //x3d_markerEvents(Data.markers.length-1);

                X3DOMObject.MapMarker.registerEvents(Lm);
                
            }
            
        }

    });
    
    Camera._map.on('mousedown',function(e){
        
        if (!e.originalEvent.ctrlKey){

            Camera._map.on('mousemove',leaf_mousemove,Camera);
            Camera._map.on('mouseup',leaf_mouseup,Camera);

        }
        
    });

}

function leaf_mouseup(e){

    var Camera = Map.marker;
    Camera._map.off('mousemove',leaf_mousemove,Camera);
    Camera._map.off('mousemove',leaf_mouseup,Camera);
    
    Camera.draggedMarker._index = null;

}

function leaf_mousemove(e){
    
    console.log("leaf mousemove");
    
    var Camera = Map.marker;
    
    // update Scene marker position
    
    if (Camera.draggedMarker._index != null){
    
        console.log(Camera.draggedMarker._latlng);
        
        var index = Camera.draggedMarker._index;
        
        var p1_ll = Camera._latlng;
        var p2_ll = Camera.draggedMarker._latlng;

        var mark = Data.markers[index];
        
        mark.latitude = p2_ll.lat;
        mark.longitude = p2_ll.lng;
        
        var azimuth = getAzimuth(p1_ll,p2_ll);
        //var initial_heading = Data.camera.heading;
        
        var angle = azimuth - INIT_HEADING_TMP;
        var distance = p1_ll.distanceTo(p2_ll);
        
        mark.x = distance*Math.sin(Math.PI/180*angle);
        mark.z = -distance*Math.cos(Math.PI/180*angle);
        
        X3DOMObject.Marker.place(mark.x,mark.y,mark.z,"my-sph-"+index);
    
    }
    
    //old
    var p0 = new L.LatLng(Data.camera.latitude,Data.camera.longitude);
    var p1 = new L.LatLng(Camera._latlng.lat,Camera._latlng.lng);
    
    Data.camera.latitude = Camera._latlng.lat;
    Data.camera.longitude = Camera._latlng.lng;
    
    var dh = Camera._heading - Math.PI/180*Data.camera.heading;
    
    //console.log(Camera._heading);
    
    Data.camera.heading = Camera._heading*180/Math.PI;
    
    var newheading = Data.camera.heading;// - INIT_HEADING;
    
    if ((p0.lat!=p1.lat)||(p0.lng!=p1.lng)){
        console.log("translation");
        leaf_translation_v1(p0,p1);
    }

    leaf_rotation_v1(newheading,dh);
    
}

function leaf_rotation_v1(newheading,angle){
    
    /* 
     * Printing values:
     * 
     *   var mat = Scene.element.runtime.viewMatrix().inverse();
     *   var rotation = new x3dom.fields.Quaternion(0, 0, 1, 0);
     *   rotation.setValue(mat);
     *   var translation = mat.e3();
     * 
     */
    
    var mat = Scene.element.runtime.viewMatrix();

    mat = mat.inverse();
    //console.log(mat.toString());

    var from = mat.e3();
    var at = from.subtract(mat.e2());
    var up = mat.e1();
    
    var q0 = x3dom.fields.Quaternion.axisAngle(up, -angle);
    var m0 = q0.toMatrix();
    
    var m1  = x3dom.fields.SFMatrix4f.translation(from);
    var m1n = x3dom.fields.SFMatrix4f.translation(from.negate());
    
    var mres = m1.mult(m0).mult(m1n);
    
    newat = mres.multMatrixPnt(at);
    
    newmat = x3dom.fields.SFMatrix4f.lookAt(from, newat, up);
    
    //if (at.y<0) at.y = -at.y;
    
    /*
    
    var alpha = Math.atan2(-at.y,Math.sqrt(at.x*at.x+at.z*at.z));
    var beta  = Math.PI/180*(newheading-INIT_HEADING_TMP);
    
    console.log("a: "+alpha+" b:"+beta);
    
    var mx = x3dom.fields.SFMatrix4f.rotationX(alpha);
    var my = x3dom.fields.SFMatrix4f.rotationY(beta);
    
    var newmat = x3dom.fields.SFMatrix4f.identity();
    
    newmat = newmat.mult(mx).mult(my);

    //newmat = mat.mult(newmat);
        
    */
        
    var Q = new x3dom.fields.Quaternion(0,0,1,0);
    //Q.setValue(newmat.inverse());
    Q.setValue(newmat);
    var AA = Q.toAxisAngle();
    
    var viewpoint = $(Scene.element).find("Viewpoint");
    viewpoint.attr("orientation",AA[0].toString()+" "+AA[1]);
    viewpoint.attr("position",from.toString());
    
}

function leaf_translation_v1(p0,p1){
    
    var pi = new L.LatLng(p0.lat,p1.lng);
    
    var dx = p0.distanceTo(pi);
    var dz = p1.distanceTo(pi);
    var dl = p0.distanceTo(p1);
    
    //console.log(dx+" "+dz+" "+dl);
    
    if (p1.lng<p0.lng){
        dx = -dx;
    }
    
    if (p1.lat<p0.lat){
        dz = -dz;
    }
    
    
    var da = Math.atan2(dz,dx);
    
    var A = Math.PI/180*INIT_HEADING_TMP;
    
    dx = dl*Math.sin(A-da);
    dz = -dl*Math.cos(A-da);
    
    var mat = Scene.element.runtime.viewMatrix().inverse();
    var tr = mat.e3();
    
    //console.log("dz = "+dz+" distance = "+dl);
    
    var x = tr.x+dx;
    var y = tr.y;
    var z = tr.z+dz;
    
    var viewpoint = $(Scene.element).find("Viewpoint");
    viewpoint.attr("position",x+" "+y+" "+z);
    
}

function x3d_mouseMove(){
    
    var Camera = Map.marker;
    
    var initial_heading = Data.camera.heading;
    //var initial_heading = INIT_HEADING;
    
    var heading = Scene.getViewDirection();
    
    Map.marker.setHeading(heading+INIT_HEADING_TMP);
    
    var d = Scene.getViewTranslation();
    
    var dx;
    var dz;
    
    if (Scene.old_view_translation == null){
        dx = d.x;
        dz = d.z;
    }else{
        dx = d.x - Scene.old_view_translation.x;
        dz = d.z - Scene.old_view_translation.z;
    }
    
    var distance = Math.sqrt(dx*dx+dz*dz);
    var angle = 180/Math.PI*Math.atan2(dx,-dz);

    var initial_coordinates = [Data.camera.latitude,Data.camera.longitude];
    
    var p0 = new L.LatLng(initial_coordinates[0],initial_coordinates[1]);//Camera._latlng;
    
    var p1 = p0.CoordinatesOf(angle+INIT_HEADING_TMP,distance);

    Map.marker.setBasePoint(p1);
    Map.marker._syncMeasureMarkersToBasePoint();

    Data.camera.latitude = p1.lat;
    Data.camera.longitude = p1.lng;
    Data.camera.heading = heading+INIT_HEADING_TMP;
    
    Scene.old_view_translation = d;
    
}

// http://www.movable-type.co.uk/scripts/latlong.html
// initial bearing

// precision problems?!

function getAzimuth2(p1,p2){

    //p1 - start point
    //p2 - end point
    
    var dlat = p2.lat-p1.lat;
    var dlon = p2.lng-p1.lng;

    var y = Math.sin(dlon)*Math.cos(p2.lat);
    var x = Math.cos(p1.lat)*Math.sin(p2.lat)-Math.sin(p1.lat)*Math.cos(p2.lat)*Math.cos(dlon);
    
    var azimuth = ((2*Math.PI + Math.atan2(y,x))*180/Math.PI) % 360;
    
    return azimuth;
  
}

function getAzimuth(p1_ll,p2_ll){
    
    var Camera = Map.marker;
    
    var p1 = Camera._map.latLngToLayerPoint(p1_ll);
    var p2 = Camera._map.latLngToLayerPoint(p2_ll);
    
    var dx = p2.x - p1.x;
    var dz = p2.y - p1.y;
    
    var azimuth = (180/Math.PI*Math.atan2(dx,-dz)+360)%360;
    
    return azimuth;
    
}


// no comments
function parseURL(){
    var parameters=location.href.replace(/\?/ig,"&").split("&");
    for (var i=0;i<parameters.length;i++) parameters[i]=parameters[i].split("=");
    for (var i=1;i<parameters.length;i++) {
        switch (parameters[i][0]) {
            case "pointer":   SETTINGS.pointer = true;break;
            case "highlight": SETTINGS.highlight = true;break;
            case "moreinfo":  SETTINGS.moreinfo = true;break;
        }
    }
}

















